<?php
/**
*
* @package smf Extension - qrlogin
* @copyright (c) 2017 qrlogin - http://qrlogin.info
* @license http://opensource.org/licenses/gpl-2.0.php GNU General Public License v2
*
*/

function qrlogin_add_actions(&$actionArray)
{
	$actionArray['qrlogin'] = array('qrlogin.php', 'qrlogin');
}

function qrlogin_add_menu_buttons(&$menu_buttons)
{
    global $scripturl, $settings;

    $menu_qrlogin = array(
    	'qrlogin' => array(
			'title' => 'qrLogin',
			'href' => $scripturl . '?action=qrlogin;qra=page',
			'show' => true,
			'icon' => '../qrlogin/qrlogin16.png',
			'sub_buttons' => array(
			),
		)
	);
    // in smf 2.0.x insert before 'login', in smf 2.1.x insert before 'signup'
    $pos = array_search('login', array_keys($menu_buttons));
    if (!$pos)
    {
        $pos = array_search('signup', array_keys($menu_buttons));
    }
    $menu_buttons = array_merge(
        array_slice($menu_buttons, 0, $pos),
        $menu_qrlogin,
        array_slice($menu_buttons, $pos)
    );
}

function qrlogin()
{
    if (!isset($_REQUEST['qra'])  || empty($_REQUEST['qra']))
    {
        qrlogin_post();
    }
    elseif ($_REQUEST['qra'] == 'ajax')
    {
        qrlogin_ajax();
    }
    elseif ($_REQUEST['qra'] == 'page')
    {
        qrlogin_page();
    }
}

function qrlogin_page()
{
	global $txt, $context, $scripturl, $smcFunc, $user_info, $modSettings;

	loadLanguage('qrlogin');
	loadTemplate('qrlogin');
	$context['sub_template'] = $user_info['is_guest'] ? 'qrlogin': 'qrlogin_user';

	// Get the template ready.... not really much else to do.
	$context['page_title'] = 'qrLogin';

	// Add the qrLogin chain to the link tree.
	$context['linktree'][] = array(
		'url' => $scripturl . '?action=qrlogin;qra=page',
		'name' => 'qrLogin',
	);

	// Set the qrLogin URL - will be used when the login process is done (but careful not to send us to an attachment).
	if ($user_info['is_guest'])
	{
    	if (isset($_SESSION['old_url']) && strpos($_SESSION['old_url'], 'dlattach') === false && preg_match('~(board|topic)[=,]~', $_SESSION['old_url']) != 0)
    		$_SESSION['login_url'] = $_SESSION['old_url'];
    	else
    		unset($_SESSION['login_url']);
	}
	
	// Saving options?
	if ($user_info['is_admin'] && isset($_GET['save']))
	{
	    qrlogin_sanitize_option('qrlogin_del_http');
	    qrlogin_sanitize_option('qrlogin_qrcode_size');
	    qrlogin_sanitize_option('qrlogin_login_timeout');
	    qrlogin_sanitize_option('qrlogin_timeout');
	    qrlogin_sanitize_option('qrlogin_poll_lifetime');
	    qrlogin_sanitize_option('qrlogin_post_timeout');
	    qrlogin_sanitize_option('qrlogin_qrcode_fore_color');
	    qrlogin_sanitize_option('qrlogin_qrcode_back_color');
	}
}

function qrlogin_sanitize_option($option)
{
    $value = $_POST[$option];
    switch ( $option ) {
        case 'qrlogin_del_http':
            if (!in_array($value, array( 0, 1 ))) {
                return;
            }
            break;
        case 'qrlogin_timeout':
            $value = intval($value);
            if (($value < 0) || ($value > 5)) {
                return; 
            }
            break;
        case 'qrlogin_poll_lifetime':
            $value = intval($value);
            if (($value < 0) || ($value > 300)) {
                return; 
            }
            break;
        case 'qrlogin_post_timeout':
            $value = intval($value);
            if (($value < 0) || ($value > 60)) {
                return; 
            }
            break;
        case 'qrlogin_login_timeout':
            $value = intval($value);
            if (($value < 0) || ($value > 10)) {
                return; 
            }
            break;
        case 'qrlogin_qrcode_size':
            $value = intval($value);
            if (($value < 64) || ($value > 256)) {
                return; 
            }
            break;
        case 'qrlogin_qrcode_fore_color':
        case 'qrlogin_qrcode_back_color':
            $value = hexdec($value);
            if($value > 0xffffff) {
                return; 
            }
            $value = "#".substr("000000".dechex($value),-6);
            break;
    }
    updateSettings( array($option => $value), true );
}

function qrlogin_exit_with_resp($resp)
{
    http_response_code($resp);
    exit;
}

function qrlogin_hack_exit()
{
    qrlogin_exit_with_resp(400);
}

function qrlogin_get_field_session($field, $sid)
{
    global $smcFunc;

	$request = $smcFunc['db_query']('', "
	    SELECT *
	    FROM {db_prefix}qrlogin
        WHERE sid = {string:sid}
        LIMIT 1",
        array(
            'sid' => $sid,
        )
	);
	if ($smcFunc['db_num_rows']($request) == 0) return;

	$res = $smcFunc['db_fetch_assoc']($request);
	$smcFunc['db_free_result']($request);
	return $res[$field];
}

function qrlogin_ajax()
{
    global $context, $modSettings, $smcFunc, $user_settings;

    // check ajax request
    if(!isset($_SERVER['HTTP_X_REQUESTED_WITH']) || empty($_SERVER['HTTP_X_REQUESTED_WITH']) || strtolower($_SERVER['HTTP_X_REQUESTED_WITH']) != 'xmlhttprequest')
        qrlogin_hack_exit();

	// check session_id in 'qrlogin_sid' 
    if (!isset($_REQUEST['qrlogin_sid']) || empty($_REQUEST['qrlogin_sid']) )
        qrlogin_hack_exit();
    $asid = preg_split( "/=/", $_REQUEST['qrlogin_sid']);
    if ($asid[0] != $context['session_id']) 
        qrlogin_hack_exit();

    $sid = md5('qrlogin' . $_REQUEST['qrlogin_sid']);
    $poll_lifetime = $modSettings['qrlogin_poll_lifetime'];

    // waiting for uid from App qrLogin - max $poll_lifetime s
    while (!$uid = qrlogin_get_field_session('uid', $sid))
    {
        if (--$poll_lifetime < 0) exit;
        sleep(1);
        if (connection_aborted()) exit;
    }
    
 	// uid received - register function for answer to App qrLogin
    register_shutdown_function('qrlogin_ajax_shutdown', $sid);

 	// Load user data
	$request = $smcFunc['db_query']('', '
		SELECT *
		FROM {db_prefix}members
		WHERE id_member = {int:id_member}
		LIMIT 1',
		array('id_member' => $uid)
	);
	// user not found !
	if ($smcFunc['db_num_rows']($request) == 0) exit;

	$user_settings = $smcFunc['db_fetch_assoc']($request);
	$smcFunc['db_free_result']($request);

	// Login user
    require_once('LogInOut.php');
    DoLogin();
}

function qrlogin_ajax_shutdown($sid)
{
    global $user_info, $smcFunc, $modSettings;

    if ($modSettings['qrlogin_post_timeout'] == 0)
    {
        // no write result - remove queue from db
        $smcFunc['db_query']('', '
            DELETE FROM {db_prefix}qrlogin
            WHERE sid = {string:sid}',
            array(
                'sid' => $sid,
            )
        );
    }
    else
    {
        // write result of login to post
        $smcFunc['db_query']('', '
            UPDATE {db_prefix}qrlogin
            SET result = {int:result}
            WHERE sid = {string:sid}',
            array(
                'result' => $user_info['is_guest'] ? 403 : 200,
                'sid' => $sid,
            )
    	);
    }
}

function qrlogin_post()
{
    global $smcFunc, $modSettings;

    // check for post from App qrLogin
    if (substr($_SERVER['HTTP_USER_AGENT'], 0, 7) != 'qrLogin')
        qrlogin_hack_exit();
 	// if data not correct
    if (!isset($_REQUEST['objectName']) || ($_REQUEST['objectName'] != 'qrLogin') ||
        !isset($_REQUEST['sessionid']) || empty($_REQUEST['sessionid']) ||
        !isset($_REQUEST['login']) || empty($_REQUEST['login']) ||
        !isset($_REQUEST['password']) || empty($_REQUEST['password']))
        qrlogin_hack_exit();

 	// Load the user data!
	$request = $smcFunc['db_query']('', '
		SELECT passwd, id_member, member_name
		FROM {db_prefix}members
		WHERE member_name = {string:user_name}
		LIMIT 1',
		array(
            'user_name' => urldecode($_REQUEST['login']),
        )
	);
	// no user found
	if ($smcFunc['db_num_rows']($request) == 0)
        qrlogin_exit_with_resp(403);

	$user_settings = $smcFunc['db_fetch_assoc']($request);
	$smcFunc['db_free_result']($request);

	// Bad password!
	if (!hash_verify_password($user_settings['member_name'], un_htmlspecialchars(urldecode($_REQUEST['password'])), $user_settings['passwd']))
        qrlogin_exit_with_resp(403);

	$sid = md5('qrlogin' . urldecode($_REQUEST['sessionid']));

	// remove queue from db
    $smcFunc['db_query']('', '
        DELETE FROM {db_prefix}qrlogin
        WHERE sid = {string:sid}',
        array(
            'sid' => $sid,
        )
    );

	// insert queue into db
	$smcFunc['db_insert']('insert',
            '{db_prefix}qrlogin',
            array(
                'sid' => 'string',
                'uid' => 'int',
            ),
            array(
                $sid, $user_settings['id_member'],
            ),
            array('sid')
        );

    // waiting for answer - max qrlogin_post_timeout s
    $post_timeout = $modSettings['qrlogin_post_timeout'];

    // if no wait answer
    if ($post_timeout == 0)
        qrlogin_exit_with_resp(200);

	while ((!$ans = qrlogin_get_field_session('result', $sid)) && ($post_timeout-- > 0))
	{
		sleep(1);
	}

	// remove queue from db
    $smcFunc['db_query']('', '
        DELETE FROM {db_prefix}qrlogin
        WHERE sid = {string:sid}',
        array(
            'sid' => $sid,
        )
    );

	// if not exists answer ! 408 Request Timeout
    qrlogin_exit_with_resp($ans ? $ans : 408 );
}

// for compatibility with smf 2.0.x
if (!function_exists('hash_verify_password')) {
    function hash_verify_password($username, $password, $hash){
        return $hash == sha1(strtolower($username) . $password);
    }
}

// for compatibility with PHP < 5.4
if (!function_exists('http_response_code')) {
    function http_response_code($code = NULL) {     
        $prev_code = (isset($GLOBALS['http_response_code']) ? $GLOBALS['http_response_code'] : 200);

        if ($code === NULL) {
            return $prev_code;
        }

        switch ($code) {
            case 100: $text = 'Continue'; break;
            case 101: $text = 'Switching Protocols'; break;
            case 200: $text = 'OK'; break;
            case 201: $text = 'Created'; break;
            case 202: $text = 'Accepted'; break;
            case 203: $text = 'Non-Authoritative Information'; break;
            case 204: $text = 'No Content'; break;
            case 205: $text = 'Reset Content'; break;
            case 206: $text = 'Partial Content'; break;
            case 300: $text = 'Multiple Choices'; break;
            case 301: $text = 'Moved Permanently'; break;
            case 302: $text = 'Moved Temporarily'; break;
            case 303: $text = 'See Other'; break;
            case 304: $text = 'Not Modified'; break;
            case 305: $text = 'Use Proxy'; break;
            case 400: $text = 'Bad Request'; break;
            case 401: $text = 'Unauthorized'; break;
            case 402: $text = 'Payment Required'; break;
            case 403: $text = 'Forbidden'; break;
            case 404: $text = 'Not Found'; break;
            case 405: $text = 'Method Not Allowed'; break;
            case 406: $text = 'Not Acceptable'; break;
            case 407: $text = 'Proxy Authentication Required'; break;
            case 408: $text = 'Request Time-out'; break;
            case 409: $text = 'Conflict'; break;
            case 410: $text = 'Gone'; break;
            case 411: $text = 'Length Required'; break;
            case 412: $text = 'Precondition Failed'; break;
            case 413: $text = 'Request Entity Too Large'; break;
            case 414: $text = 'Request-URI Too Large'; break;
            case 415: $text = 'Unsupported Media Type'; break;
            case 500: $text = 'Internal Server Error'; break;
            case 501: $text = 'Not Implemented'; break;
            case 502: $text = 'Bad Gateway'; break;
            case 503: $text = 'Service Unavailable'; break;
            case 504: $text = 'Gateway Time-out'; break;
            case 505: $text = 'HTTP Version not supported'; break;
            default:
                trigger_error('Unknown http status code ' . $code, E_USER_ERROR); // exit('Unknown http status code "' . htmlentities($code) . '"');
                return $prev_code;
        }

        $protocol = (isset($_SERVER['SERVER_PROTOCOL']) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0');
        header($protocol . ' ' . $code . ' ' . $text);
        $GLOBALS['http_response_code'] = $code;

        // original function always returns the previous or current code
        return $prev_code;
    }
}
